package graphs.amgraph;

import java.util.*; 
import graphs.*;

/**
 * Adjacency Matrix implementation of a graph
 *
 * @author V.B.
 * @version 2.0.0 Dec 2001
 */
public class GraphAM implements Graph {
  
	/** graphEdges[i][j] stores the edge between vertices i and j. */
  protected Edge[][] graphEdges;  
  protected ArrayList vertices;
  
  /** Contains the number of edges in this graph */
  protected int numEdges;
  protected int numVertices;
           
  /**
   * Constructs an empty graph.
   */         
  public GraphAM() {
  	this(new ArrayList());
  }
  
  /**
   * Constructs a graph using the passed matrix representation.
   *
   * @param vertices  the graph vertices
   */
  public GraphAM(ArrayList vertices) {
    this.numVertices = vertices.size();
    this.vertices = vertices;
    numEdges = 0;
    graphEdges = new Edge[numVertices][numVertices];
  }
  
  /* 1.General methods - return global information about the graph */
  
  /** Returns the numers of vertices in this graph. */
  public int numVertices() {
    return vertices.size();
  }
  
  /** Returns the number of edges in this graph.*/
  public int numEdges() { 
    return numEdges;
  }

  /** Returns an iterator of the vertex positions in this graph. */  
  public Iterator vertices() {
    return vertices.iterator();
  }

  /** Returns an iterator of the edge positions in this graph. */  
  public Iterator edges() { 
    ArrayList edges = new ArrayList();
    for (int i = 0; i < graphEdges.length; i++) 
      for (int j = 0; j < graphEdges[i].length; j++)
        edges.add(graphEdges[i][j]);
    return edges.iterator();
  }
  
      
  /* Methods that take vertex and edge positions as arguments: */
      
  /** Returns the degree of the vertex v. */  
  public int degree(Vertex v) {
  	int result = 0;
  	int rank = ((VertexAM) v).getRank();
    for (int i = 0; i < numVertices; i++) {
    	if (graphEdges[rank][i] != null) {
      	result++;
    	}
    	if ((graphEdges[i][rank]!=null) && (i!=rank)) {
    		result++;
    	}
    }
    return result;
  }
  
  /* Methods that take vertex and edge positions as arguments: */
      
  /** Returns an iterator of the vertices adjacent to v. */
  public Iterator adjacentVertices(Vertex v) {
  	int n = ((VertexAM) v).getRank();
  	Collection c = getInAdjVertices(n);
  	c.addAll(getOutAdjVertices(n));
  	return c.iterator();
  }

  /** Returns an iterator of the edges incident upon v. */  
  public Iterator incidentEdges(Vertex v) {
    int n = ((VertexAM) v).getRank();
  	Collection c = getInIncidentEdges(n);
    c.addAll(getOutIncidentEdges(n));
  	return c.iterator();
  }
  
  
  /** Returns an array of size 2 storing the end vertices of e. */
  public Vertex[] endVertices(Edge e) {
    Vertex[] result = null;
  	for(int i = 0; i < numVertices; i++) {
  		for(int j = 0; j < numVertices; j++) {
    		if ((graphEdges[i][j]!=null) && (graphEdges[i][j].equals(e))) {
    			result = new Vertex[2];
    			result[1] = (Vertex)vertices.get(i);
    			result[2] = (Vertex)vertices.get(j);
    			break;
    		}
  		}
    }
  	return result;
  }

  /** Returns the endpoint of edge e distinct from v. */  
  public Vertex opposite(Vertex v, Edge e) {
  	Vertex result = null;
  	int rank = ((VertexAM) v).getRank();
  	for(int i = 0; i < numVertices; i++) {
  		if ((graphEdges[rank][i]!=null) && (graphEdges[rank][i].equals(e))) {
  			result = (Vertex) vertices.get(i);
  			break;
  		}
  	}
    return result;
  }

  /** Checks whether the vertices 'v' and 'w' are adjacent. */
  public boolean areAdgacent(Vertex v, Vertex w) {
    int i = vertices.indexOf(v);
    int j = vertices.indexOf(w);
    return !(graphEdges[i][j] == null);
  }
	
  
  public void insertEdge(int i, int j, Object o) {
  	graphEdges[i][j] = new Edge(o);
  	numEdges++;
  }
  
  /** Inserts an undirected edge between the i-th and j-th vertices */
  public void insertEdge(int i, int j) {
		insertEdge(i, j, null);
  }

  
  public void setVertex(int rank, Vertex v) {
  	vertices.set(rank, v);
  }
  
  /** Removes edge e. */
  public void removeEdge(Edge e) {
  	for(int i = 0; i < numVertices; i++) {
  		for(int j = 0; j < numVertices; j++) {
    		if ((graphEdges[i][j]!=null) && (graphEdges[i][j].equals(e))) {
    			graphEdges[i][j]=null;
    			numEdges--;
    			break;
    		} 
      }
  	}
  }
	
  private Collection getInAdjVertices(int rank) {

  	ArrayList adjacent = new ArrayList();
  	
    for (int i = 0; i < numVertices; i++) {
      if (graphEdges[rank][i] != null) {
        adjacent.add(vertices.get(i));
      }
    }
    return adjacent;
  }
  
  private Collection getOutAdjVertices(int rank) {
  	ArrayList adjacent = new ArrayList();
  	
    for (int i = 0; i < numVertices; i++) {
      if (graphEdges[i][rank] != null) {
        adjacent.add(vertices.get(i));
      }
    }
    return adjacent;
  }

  private Collection getInIncidentEdges(int rank) {
  	ArrayList incident = new ArrayList();
  	
    for (int i = 0; i < numVertices; i++) {
      if (graphEdges[rank][i] != null) {
        incident.add(graphEdges[rank][i]);
      }
    }
    return incident;
  }
  
  private Collection getOutIncidentEdges(int rank) {
  	ArrayList incident = new ArrayList();
  	
    for (int i = 0; i < numVertices; i++) {
      if (graphEdges[i][rank] != null) {
        incident.add(graphEdges[i][rank]);
      }
    }
    return incident;
  }

  
}//to do: error conditions that may occur